import { EmptyObj, WSEndpoint } from '../websocket';

export enum LobbyMessageType {
    CREATE = 'lobby.create',
    JOIN = 'lobby.join',
    LEAVE = 'lobby.leave',
    DATA_OFFER = 'lobby.data-offer',
    DATA_ANSWER = 'lobby.data-answer',
    ICE_CANDIDATE = 'lobby.ice-candidate'
}
export type LobbyUserObj = {
    id: string,
    name: string,
}
export type LobbyJoinerObj = {
    token: string
} & LobbyUserObj;

type LobbyId = {
    lobbyId: string,
}
type LobbyRecipientData = {
    to: string,
}
type LobbyExchangeMessageData = {
    from: string,
} & LobbyRecipientData;
type LobbyJoinerData = {
    user?: LobbyJoinerObj,
}
type LobbyUsersData = {
    users: LobbyUserObj[],
}
type LobbySessionDescriptionData = {
    description: RTCSessionDescriptionInit,
}
type LobbyIceCandidateData = {
    candidate: RTCIceCandidate,
}

export type LobbyAPI = {
    [LobbyMessageType.CREATE]: WSEndpoint<
        EmptyObj,
        LobbyId
    >
    [LobbyMessageType.JOIN]: WSEndpoint<
        LobbyId,
        LobbyId & LobbyJoinerData & LobbyUsersData
    >
    [LobbyMessageType.LEAVE]: WSEndpoint<
        LobbyId,
        LobbyId & LobbyUsersData
    >
    [LobbyMessageType.DATA_OFFER]: WSEndpoint<
        LobbyId & LobbyRecipientData & LobbySessionDescriptionData,
        LobbyId & LobbyExchangeMessageData & LobbySessionDescriptionData
    >
    [LobbyMessageType.DATA_ANSWER]: WSEndpoint<
        LobbyId & LobbyRecipientData & LobbySessionDescriptionData,
        LobbyId & LobbyExchangeMessageData & LobbySessionDescriptionData
    >
    [LobbyMessageType.ICE_CANDIDATE]: WSEndpoint<
        LobbyId & LobbyRecipientData & LobbyIceCandidateData,
        LobbyId & LobbyExchangeMessageData & LobbyIceCandidateData
    >
}