import { v4 as uuidv4 } from 'uuid';
import { LobbyModel } from 'wetube-common';
import { IUserRepository, ILobbyRepository } from '../repositories';

export class LobbyService {
    private lobbyRepository: ILobbyRepository;
    private userRepository: IUserRepository;

    constructor(userRepository: IUserRepository, lobbyRepository: ILobbyRepository) {
        this.userRepository = userRepository;
        this.lobbyRepository = lobbyRepository;
    }

    public createLobby() {
        const id = uuidv4();
        const lobby = new LobbyModel(id);
        this.lobbyRepository.save(lobby);

        return lobby;
    }

    public getLobby(lobbyId: string) {
        const lobby = this.lobbyRepository.getById(lobbyId);
        return lobby;
    }

    public joinLobby(lobbyId: string, userId: string) {
        const lobby = this.lobbyRepository.getById(lobbyId);
        const user = this.userRepository.getById(userId);
        if (!lobby || !user) {
            return;
        }

        lobby.users.push(user.id);
        this.lobbyRepository.update(lobby);

        user.lobbies.push(lobby.id);
        this.userRepository.update(user);

        return lobby;
    }

    public leaveLobby(lobbyId: string, userId: string) {
        const lobby = this.lobbyRepository.getById(lobbyId);
        const user = this.userRepository.getById(userId);
        if (!lobby || !user) {
            return;
        }

        lobby.users = lobby.users.filter(id => id != user.id);
        this.lobbyRepository.update(lobby);

        user.lobbies = user.lobbies.filter(id => id != lobby.id);
        this.userRepository.update(user);

        return lobby;
    }

    public leaveAllLobbies(userId: string) {
        const user = this.userRepository.getById(userId);
        if (!user) {
            return [];
        }

        const affectedLobbies = user.lobbies
            .map(lobbyId => this.lobbyRepository.getById(lobbyId))
            .filter((lobby): lobby is LobbyModel => lobby != null);

        affectedLobbies.forEach(lobby => {
            lobby.users = lobby.users.filter(id => id != user.id);
            this.lobbyRepository.update(lobby);
        });

        user.lobbies = [];
        this.userRepository.update(user);

        return affectedLobbies;
    }

}