import { v4 as uuidv4 } from 'uuid';
import { UserModel } from 'wetube-common';
import { IUserRepository } from '../repositories';
import { AuthDataPayload } from '../contract';
import { ITokenAuth } from '../utils/token';
import { RandomNameGenerator } from '../utils/random';

export class UserService {
    private tokenAuth: ITokenAuth<AuthDataPayload>;
    private userRepository: IUserRepository;
    private randomNameGenerator: RandomNameGenerator;

    private secret = 'test-secret';

    constructor(tokenAuth: ITokenAuth<AuthDataPayload>, userRepository: IUserRepository, randomNameGenerator: RandomNameGenerator) {
        this.tokenAuth = tokenAuth;
        this.userRepository = userRepository;
        this.randomNameGenerator = randomNameGenerator;
    }

    public createUser(): UserModel {
        const id = uuidv4();
        const token = this.tokenAuth.sign({
            id,
            // 24 hours
            exp: Math.floor(Date.now() / 1000) + (60 * 60 * 24)
        }, this.secret);
        const user = new UserModel(id, this.randomNameGenerator.getName(), token);

        this.userRepository.save(user);

        return user;
    }

    public getUser(id: string) {
        return this.userRepository.getById(id);
    }

    public validateUser(token: string): AuthDataPayload {
        return this.tokenAuth.verify(token, this.secret);
    }

}