import { LobbyAPI, LobbyMessageType } from 'wetube-common';
import { AppApi } from './AppApi';
import { WSClientConnection } from '../utils';
import { InMemoryAuthDataServices } from '../services/auth-data';
import { LobbyService } from '../services/lobby';
import { MediaSyncService } from '../services/media-sync';
import { MediaMetadataService } from '../services/media-metadata';

export const createAppApi = () => {
    const authDataService = new InMemoryAuthDataServices();

    const socketUrl = DEBUG ? 'ws:localhost:3000' : 'wss:/wetube-signal.azurewebsites.net';
    const socketClient = new WSClientConnection<LobbyAPI>(socketUrl, {
        getAuthToken: () => authDataService.getData()?.token ?? null,
        authEndpoints: [LobbyMessageType.JOIN]
    });
    socketClient.connect();

    const lobbyService = new LobbyService(socketClient, authDataService);
    const mediaSyncService = new MediaSyncService(lobbyService);
    const mediaMetadataService = new MediaMetadataService();

    return new AppApi({
        authDataService,
        lobbyService,
        mediaSyncService,
        mediaMetadataService,
    });

};