import React from 'react';
import { styled } from '@mui/material/styles';
import Paper from '@mui/material/Paper';
import Box from '@mui/material/Box';
import InputBase from '@mui/material/InputBase';
import IconButton from '@mui/material/IconButton';
import Icon from '@mui/material/Icon';
import Collapse from '@mui/material/Collapse';
import { ClickableVideoItem } from './VideoItem';

import { useMediaSearch } from '../stores';
import { MediaMetadata } from '../../../services/media-metadata';

const StyledContainer = styled(Box)(({ theme }) => ({
    display: 'flex',
    justifyContent: 'center',
    width: 500,
    [theme.breakpoints.down('md')]: {
        width: '100%'
    },
}));

const StyledSearchContainer = styled(Paper)(({ theme }) => ({
    position: 'relative',
    display: 'flex',
    justifyContent: 'space-between',
    width: 500,
    [theme.breakpoints.down('md')]: {
        width: '100%'
    },
}));

const StyledInput = styled(InputBase)({
    '& .MuiInputBase-input': {
        paddingTop: 10,
        paddingBottom: 10,
    },
    marginLeft: 15,
    marginRight: 15,
    flex: 1,
});

const StyledClearButton = styled(IconButton)({
    padding: 10,
});

const StyledSuggestionContainer = styled(Paper)({
    position: 'absolute',
    width: '100%',
    top: '100%',
    marginTop: 5,
    zIndex: 1,
});

interface VideoSearchProps {
    className?: string;
    onSuggestionPick: (metadata: MediaMetadata) => void;
}
export const VideoSearch = ({ className, onSuggestionPick }: VideoSearchProps) => {
    const {
        mediaMetadata, setMediaMetadata,
        searchQuery, setSearchQuery
    } = useMediaSearch();

    const clearSearch = () => {
        setSearchQuery('');
        setMediaMetadata(null);
    };

    const pickSuggestionMetadata = (metadata: MediaMetadata) => {
        onSuggestionPick(metadata);
        clearSearch();
    };

    const clearSearchButton = searchQuery ? (
        <StyledClearButton
            type='button'
            aria-label='search'
            onClick={clearSearch}
        >
            <Icon>{'close'}</Icon>
        </StyledClearButton>
    ) : null;

    return (
        <StyledContainer>
            <StyledSearchContainer
                component='div'
                className={className}
            >
                <StyledInput
                    placeholder='Paste YouTube video link'
                    inputProps={{ 'aria-label': 'Paste YouTube video link' }}
                    value={searchQuery}
                    onChange={(ev: React.ChangeEvent<HTMLInputElement>) => setSearchQuery(ev.target.value)}
                />
                {clearSearchButton}
                <StyledSuggestionContainer>
                    <Collapse
                        in={mediaMetadata != null}
                    >
                        <ClickableVideoItem
                            metadata={mediaMetadata}
                            onClick={pickSuggestionMetadata}
                        />
                    </Collapse>
                </StyledSuggestionContainer>
            </StyledSearchContainer>
        </StyledContainer>
    );
};