const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CopyPlugin = require('copy-webpack-plugin');

const TypescriptLoader = {
    test: /\.tsx?$/,
    use: 'ts-loader',
    exclude: /node_modules/,
};
const StyleLoader = {
    test: /\.css$/i,
    use: [
        "style-loader",
        "css-loader",
    ],
}

module.exports = {
    entry: './src/wetube-web.tsx',
    module: {
        rules: [
            TypescriptLoader,
            StyleLoader,
        ],
    },
    plugins: [
        new HtmlWebpackPlugin({
            template: path.resolve(__dirname, 'public/index.html'),
        }),
        new CopyPlugin({
            patterns: [
                { from: '*.css', context: 'public/' },
            ]
        }),
    ],
    resolve: {
        extensions: ['.tsx', '.ts', '.js'],
        alias: {
            '@mui/styled-engine': '@mui/styled-engine-sc',
        },
    },
    output: {
        filename: 'wetube-web.js',
        path: path.resolve(__dirname, 'lib/public')
    },
};